/*


  Simple Demo for GLSL

  www.lighthouse3d.com

  Cleaned up, modified, and re-factored by GZ, June 2007

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(__linux__)

#define GL_GLEXT_PROTOTYPES
#include "gl.h"
#include <glu.h>
#include <glut.h>

#elif defined(__APPLE__)

#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>

#endif

#include "shader_setup.h"

#include "trackball.h"  // trackball quaternion routines

// trackbal state
int spinning = 0, moving = 0;
int beginx, beginy;
int W = 300, H = 300;
float curquat[4];
float lastquat[4];

GLuint sh_prog_id;

float lpos[4] = {1.0,0.0,1.0,0.0};

// rendering

void changeSize(int w, int h) {

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if ( h == 0 )
		h = 1;

	float ratio = 1.0* w / h;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	// Set the correct perspective.
	gluPerspective(45,ratio,1,100);
	glMatrixMode(GL_MODELVIEW);


}

float a = 0;

void renderScene(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0,5.0,5.0, 
		      0.0,0.0,0.0,
			  0.0f,1.0f,0.0f);

	// trackball
	GLfloat m[4][4];
	build_rotmatrix(m, curquat);
	glMultMatrixf(&m[0][0]);

	glLightfv(GL_LIGHT0, GL_POSITION, lpos);
	glRotatef(a,0,1,0);

	glutSolidTeapot(1);
	a+=0.1;

	glutSwapBuffers();
}


void processNormalKeys(unsigned char key, int x, int y)
{
	if (key == 27) 
		exit(0);
}


// trackball

void
mouse(int button, int state, int x, int y)
{
  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
    spinning = 0;
    //glutIdleFunc(NULL);
	glutIdleFunc(renderScene);
    moving = 1;
    beginx = x;
    beginy = y;
  }
  if (button == GLUT_LEFT_BUTTON && state == GLUT_UP) {
    moving = 0;
  }
}

void
animate(void)
{
  add_quats(lastquat, curquat, curquat);
  glutPostRedisplay();
}

void
motion(int x, int y)
{
  if (moving) {
    trackball(lastquat,
      (2.0 * beginx - W) / W,
      (H - 2.0 * beginy) / H,
      (2.0 * x - W) / W,
      (H - 2.0 * y) / H
      );
    beginx = x;
    beginy = y;
    spinning = 1;
    glutIdleFunc(animate);
  }
}



int getUniLoc( const char * uni_name )
{
	GLint uni_id = glGetUniformLocation( sh_prog_id, uni_name );
	if ( uni_id == -1 )
		fprintf(stderr,"uniform 'time' does not exist!\n");
	return uni_id;
}


// main 

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB );
	glutInitWindowPosition(100,100);
	glutInitWindowSize(320,320);
	glutCreateWindow("Toon Shading");

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);

	// trackball
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	trackball(curquat, 0.0, 0.0, 0.0, 0.0);

	glEnable(GL_DEPTH_TEST);
	glClearColor(1.0,1.0,1.0,1.0);
	// glEnable(GL_CULL_FACE);		// this makes the teapot look funny

	checkShaderExt();

	sh_prog_id = setShaders( "toon.vert", "toon.frag" );

	GLint color = getUniLoc( "color" );
	glUniform3f( color, 1.0, 0.2, 0.5 );

	glutMainLoop();

	return 0;
}

