// Procedural brick texture; inspired by ch. 6 in the orange book
// Includes noise, but use noise texture.

uniform sampler2D tex;				// use noise texture instead of noise function!
// try textures/noise.jpg

varying vec4 LightIntensity;		// contains diffuse and specular terms, w/o object color
varying vec2 Pos_mc;				// the position of the vertex/fragment in model(!) coords


uniform vec4 BrickColor, MortarColor;
uniform vec2 BrickStepSize, BrickPercent;

uniform float noise_spatial_freq;	// spatial frequency of the noise
uniform float noise_amplitude;		// use starting values: f = 0.3  a = 0.3
uniform float noise_time_shift;		// doesn't make sense here, but for demo purposes

uniform float noise_spatial_freq2;	// fine-grain noise
uniform float noise_amplitude2;		// use starting values: f = 5  a = 0.1


// returns a noise in the range [-1,1]
float mynoise1( in vec2 p )
{
	return 2.0 * texture2D( tex, p ).r  - 1.0;
}


// returns a noise vector in the range [-1,1]^3
vec3 mynoise3( in vec2 p )
{
	return 2.0 * texture2D( tex, p ).rgb - 1.0;
}


void main()
{
	vec2 pos = Pos_mc / BrickStepSize;

	// are we in an odd brick row?
	if ( mod(abs(pos.y), 2.0) > 1.0 )		// this saves conversion to int / mod
		pos.x += 0.5;

	pos = fract( pos );						// = rel. position within brick

	// determine color
	vec4 color;
	vec4 dirt;
	if ( any( greaterThan(pos, BrickPercent) ) )
	{
		color = MortarColor;
		// add some "dirt"
		float c = noise_amplitude * mynoise1( noise_spatial_freq * vec2(Pos_mc) + 
									 	 vec2(noise_time_shift) );
		c += noise_amplitude2 * mynoise1( noise_spatial_freq2 * vec2(Pos_mc) + 
									  vec2(noise_time_shift) );
		dirt = vec4( c, c, c, 1.0 );
	}
	else
	{
		color = BrickColor;
		// add some "dirt"
		vec3 c = noise_amplitude * mynoise3( noise_spatial_freq * vec2(Pos_mc) + 
										  vec2(noise_time_shift) );
		c += noise_amplitude2 * mynoise3( noise_spatial_freq2 * vec2(Pos_mc) + 
									  vec2(noise_time_shift) );
		dirt = vec4( c, 1.0 );	
	}
	color += dirt;


	color *= LightIntensity;
	gl_FragColor = color;
}
